@@ -148,6 +148,10 @@ fit_default <- function(mod, aggregate, optimizer, quiet, start_oldpar) {
 fit_inner_outer <- function(mod, optimizer, quiet, vars_inner, start_oldpar) {
   if (start_oldpar)
     cli::cli_abort("{.arg start_oldpar} must be {.val {FALSE}} when using \"inner-outer\" method.")
+  if (has_covariates(mod))
+    cli::cli_abort("\"inner-outer\" method cannot be used with models that include covariates.")
+  if (has_datamod(mod))
+    cli::cli_abort("\"inner-outer\" method cannot be used with models that include a data model.")
   if (is.null(vars_inner))
     vars_inner <- make_vars_inner(mod)
   else
@@ -247,13 +251,13 @@ make_fit_data <- function(mod, aggregate) {
   dimnames_terms <- mod$dimnames_terms
   terms_effect <- make_terms_effects(dimnames_terms)
   has_covariates <- has_covariates(mod)
-  i_lik <- make_i_lik_mod(mod) ## index of function to use for calculating likelihood in TMB
+  i_lik <- make_i_lik(mod)
   terms_effectfree <- make_terms_effectfree(mod)
   uses_matrix_effectfree_effect <- make_uses_matrix_effectfree_effect(mod)
   matrices_effectfree_effect <- make_matrices_effectfree_effect(mod)
   uses_offset_effectfree_effect <- make_uses_offset_effectfree_effect(mod)
   offsets_effectfree_effect <- make_offsets_effectfree_effect(mod)
-  i_prior <- make_i_prior(mod) ## index of function to use for calculating prior density in TMB
+  i_prior <- make_i_prior(mod)
   uses_hyper <- make_uses_hyper(mod)
   terms_hyper <- make_terms_hyper(mod)
   uses_hyperrandfree <- make_uses_hyperrandfree(mod)
@@ -262,6 +266,9 @@ make_fit_data <- function(mod, aggregate) {
   terms_const <- make_terms_const(mod)
   matrices_along_by_effectfree <- make_matrices_along_by_effectfree(mod)
   mean_disp <- mod$mean_disp
+  i_datamod <- make_fit_i_datamod(mod)
+  datamod_consts <- make_fit_datamod_consts(mod)
+  datamod_matrices <- make_fit_datamod_matrices(mod)
   list(i_lik = i_lik,
        outcome = outcome,
        offset = offset,
@@ -281,10 +288,101 @@ make_fit_data <- function(mod, aggregate) {
        terms_consts = terms_const,
        matrices_along_by_effectfree = matrices_along_by_effectfree,
        mean_disp = mean_disp,
-       matrix_covariates = matrix_covariates)
+       matrix_covariates = matrix_covariates,
+       i_datamod = i_datamod,
+       datamod_consts = datamod_consts,
+       datamod_matrices = datamod_matrices)
 }
 
 
+## HAS_TESTS
+#' Make Vector Holding Constants for Data Model
+#'
+#' Return vector of length 0 if no data model
+#'
+#' @param mod Object of class 'bage_mod'
+#'
+#' @returns A double vector
+#'
+#' @noRd
+make_fit_datamod_consts <- function(mod) {
+  has_datamod <- has_datamod(mod)
+  if (has_datamod) {
+    datamod <- mod$datamod
+    ans <- make_datamod_consts(datamod)
+  }
+  else
+    ans <- double()
+  ans
+}
+
+
+## HAS_TESTS
+#' Make List Holding Matrices for Data Model
+#'
+#' Return list of length 0 if no data model
+#'
+#' @param mod Object of class 'bage_mod'
+#'
+#' @returns A list
+#'
+#' @noRd
+make_fit_datamod_matrices <- function(mod) {
+  has_datamod <- has_datamod(mod)
+  if (has_datamod) {
+    datamod <- mod$datamod
+    ans <- make_datamod_matrices(datamod)
+  }
+  else
+    ans <- list()
+  ans
+}
+
+
+## HAS_TESTS
+#' Make Vector Holding Parameters for Data Model
+#'
+#' Return vector of length 0 if no data model,
+#' or data model does not use parameters
+#'
+#' @param mod Object of class 'bage_mod'
+#'
+#' @returns A list
+#'
+#' @noRd
+make_fit_datamod_param <- function(mod) {
+  has_datamod <- has_datamod(mod)
+  if (has_datamod) {
+    datamod <- mod$datamod
+    ans <- make_datamod_param(datamod)
+  }
+  else
+    ans <- double()
+  ans
+}
+
+
+## HAS_TESTS
+#' Make 'i_datamod', Index for Data Models
+#'
+#' Value is 0 if no data model
+#'
+#' @param mod Object of class 'bage_mod'
+#'
+#' @returns An integer scalar
+#'
+#' @noRd
+make_fit_i_datamod <- function(mod) {
+  has_datamod <- has_datamod(mod)
+  if (has_datamod) {
+    datamod <- mod$datamod
+    ans <- make_i_lik_part(datamod)
+  }
+  else
+    ans <- 0L
+  ans
+}
+      
 
 ## HAS_TESTS
 #' Make mapping used by MakeADFun
@@ -300,12 +398,13 @@ make_fit_data <- function(mod, aggregate) {
 #'
 #' @noRd
 make_fit_map <- function(mod) {
+  eps_disp <- 1e-6
   priors <- mod$priors
   mean_disp <- mod$mean_disp
   ## determine whether any parameters fixed
   is_known <- vapply(priors, is_known, FALSE)
   is_effectfree_fixed <- any(is_known)
-  is_disp_fixed <- mean_disp == 0
+  is_disp_fixed <- abs(mean_disp) < eps_disp
   ## return NULL if nothing fixed
   if (!is_effectfree_fixed && !is_disp_fixed)
     return(NULL)
@@ -333,18 +432,21 @@ make_fit_parameters <- function(mod) {
   hyperrandfree <- make_hyperrandfree(mod)
   log_disp <- c(disp = 0)
   coef_covariates <- make_coef_covariates(mod)
+  datamod_param <- make_fit_datamod_param(mod)
   list(effectfree = effectfree,
        hyper = hyper,
        hyperrandfree = hyperrandfree,
        log_disp = log_disp,
-       coef_covariates = coef_covariates)
+       coef_covariates = coef_covariates,
+       datamod_param = datamod_param)
 }
 
 
 ## HAS_TESTS
 #' Make 'random' argument to MakeADFun function
 #'
-#' Return value always includes "effectfree".
+#' Need to make sure there are at least
+#' some fixed effects
 #'
 #' @param mod Object of class "bage_mod"
 #'
@@ -356,15 +458,17 @@ make_fit_random <- function(mod) {
   has_hyper <- any(make_lengths_hyper(mod) > 0L)
   has_hyperrandfree <- any(vapply(priors, has_hyperrandfree, FALSE))
   has_covariates <- has_covariates(mod)
-  if (!has_hyper && !has_hyperrandfree && !has_covariates)
-    ans <- NULL
-  else {
+  has_datamod_param <- has_datamod_param(mod)
+  if (has_hyper)
     ans <- "effectfree"
-    if (has_hyperrandfree)
-      ans <- c(ans, "hyperrandfree")
-    if (has_covariates)
-      ans <- c(ans, "coef_covariates")
-  }
+  else
+    ans <- NULL
+  if (has_hyperrandfree)
+    ans <- c(ans, "hyperrandfree")
+  if (has_covariates)
+    ans <- c(ans, "coef_covariates")
+  if (has_datamod_param)
+    ans <- c(ans, "datamod_param")
   ans
 }
 
@@ -442,8 +546,8 @@ optimize_adfun <- function(f,
     out <- optimize_cg(f = f, quiet = quiet)
   else
     cli::cli_abort("Internal error: {.val {optimizer}} is not a valid value for {.arg optimizer}.")
-  if (!out$converged)
-    cli::cli_alert_warning("Optimizer did not converge.")
+  if (!out$converged) # nocov
+    cli::cli_alert_warning("Optimizer did not converge.") # nocov
   out
 }
 
